import click
from mcserverapi import Server, Parser
from .bot import MC, MCBot
import threading


class DiscordMCParser(Parser):
    def __init__(self, bot, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.bot = bot

    def on_ready(self, ctx):
        self.bot.announce('Server started in {}'.format(ctx[0]))

    def on_player_message(self, ctx):
        player, message = ctx
        self.bot.message_queue[message] = {
            'sent': False,
            'player': player
        }



@click.command(name='mcs', help='Runs the server')
@click.argument('jar_path')
@click.argument('token')
@click.argument('channel_id')
def cli(jar_path, token, channel_id):
    server = Server(jar_path)
    server.start()

    bot = MCBot('m!')
    parser = DiscordMCParser(bot, server)
    threading.Thread(target=parser.wait_for_events).start()

    bot.add_cog(MC)
    bot.server = server
    bot.channel_id = channel_id
    bot.run(token)






from discord.ext.commands import command, Cog, Bot
import threading


class MCBot(Bot):
    async def on_message(self, message):
        if message.author != self.user:
            if not message.content.startswith(self.command_prefix):
                self.server.run_cmd('execute', 'as', message.author.nick, 'run', 'say', message.content)
            else:
                await self.process_commands(message)


class MC(Cog):
    def __init__(self, bot):
        self.bot = bot
        self.bot.message_queue = {}
        threading.Thread(target=self.bot.loop.call_soon_threadsafe, args=[self.bot.message_queue]).start()

    @command(name='mc')
    async def mc(self, ctx, cmd, *params):
        self.bot.server.run_cmd('execute', 'as', ctx.author.nick, 'run', cmd, *params)

    async def message_worker(self):
        while self.bot.server.online:
            for message in self.bot.message_queue:
                if not self.bot.message_queue[message]['sent']:
                    await self.bot.get_channel(self.bot.channel_id).send(
                        '``' + self.bot.message_queue[message]['player'] + ' > ' + message + '``'
                    )
                    self.bot.message_queue[message]['sent'] = True



from setuptools import setup

setup(
    name='DiscordMC',
    version='0.0.3',
    packages=['discordmc'],
    license='GNU LIcense',
    author='Pixymon',
    author_email='nlarsen23.student@gmail.com',
    description='Access to your Minecraft Server through discord.',
    install_requires=['click', 'discord', 'mcserverapi'],
    entry_points={
        'console_scripts': ['discordmc=discordmc.cli:cli'],
    }
)
